"""
m_stats: A statistics and probability library for Python
=======================================================

"""
from m_stats import anova
from m_stats import distributions
from m_stats import generals
from m_stats import hypothesis
from m_stats import intervals
from m_stats import categorical
from m_stats import linregr
from m_stats import graph
from m_stats import multiregr
from m_stats import timeseries
